﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace BlogPageFlipStudy04
{
    public partial class MainPage : UserControl
    {
        double pageWidth = 250;
        double pageHeight = 300;
        double pageHalfHeight;
        // I'M DEFINING SOME COLORS EARLY TO SAVE TIME LATER...
        Color red = Color.FromArgb(0x66, 0xff, 0x00, 0x00);
        Color green = Color.FromArgb(0xaa, 0x00, 0x99, 0x00);
        Color blue = Color.FromArgb(0xaa, 0x00, 0x00, 0xff);
        Color purple = Color.FromArgb(0xFF, 0x44, 0x11, 0x44);
        Color grey = Color.FromArgb(0x44, 0x00, 0x00, 0x00);

        public MainPage()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        Point origin;

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            // MOST OF THE LOADED FUNCTION IS USED TO SET LOOK UP VALUES
            // BASED ON THE pageWidth AND pageHeight INITIALLY DEFINED.
            pageHalfHeight = pageHeight * .5;
            origin = new Point(pageWidth, pageHeight / 2.0);
            Dots.SetValue(Canvas.LeftProperty, origin.X);
            Dots.SetValue(Canvas.TopProperty, origin.Y);
            spineBottom = new Point(0, pageHalfHeight);
            spineTop = new Point(0, -pageHalfHeight);
            fixedRadius = pageWidth;
            pageDiagonal = Math.Sqrt(pageWidth * pageWidth + pageHeight * pageHeight);
            // POSITION THE PAGE AND SPINE VISUALS ACCORDINGLY
            layoutRoot.Width = pageWidth * 2;
            layoutRoot.Height = pageHeight;
            lineVertical.X1 = lineVertical.X2 = pageWidth;
            lineVertical.Y1 = 0;
            lineVertical.Y2 = pageHeight;
            hotCorner.SetValue(Canvas.LeftProperty, pageWidth * 2 - 50);
            hotCorner.SetValue(Canvas.TopProperty, pageHeight - 50);
            mouse.X = pageWidth - 1;
            mouse.Y = pageHalfHeight - 1;
            // THE ONLY EVENT HANDLER YOU NEED IS TO ACTIVE MOUSE FOLLOWING ON THE HOTCORNER
            hotCorner.MouseLeftButtonDown += new MouseButtonEventHandler(hotCorner_MouseLeftButtonDown);
            hotCorner.MouseLeftButtonUp += new MouseButtonEventHandler(hotCorner_MouseLeftButtonUp);
            hotCorner.MouseMove += new MouseEventHandler(hotCorner_MouseMove);
            hotCorner.MouseLeave += new MouseEventHandler(hotCorner_MouseLeave);

            // INITIALIZE THE MAIN CLIPPING RECTANGLE
            maskSize.X = pageWidth;
            maskSize.Y = pageHeight * 1.6;
            mainMask.Width = maskSize.X;
            mainMask.Height = maskSize.Y;
            mainMask.SetValue(Canvas.TopProperty, pageHeight - maskSize.Y * .85);

            // CREATE YOUR ANIMATION LOOP
            CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
        }

        // BASIC MOUSE CAPTURING OVER THE HOT CORNER.
        // IF YOU LET GO OF THE MOUSE BEFORE CROSSING THE SPINE, FOLLOW RETURNS TO THE RIGHT EDGE BOTTOM.
        // IF YOU LET GO OF THE MOUSE AFTER CROSSING THE SPINE, FOLLOW FLIPS TO THE LEFT EDGE BOTTOM.
        void hotCorner_MouseLeave(object sender, MouseEventArgs e)
        {
            mouse = new Point(pageWidth - 1, pageHalfHeight - 1);
        }

        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            updateUI();
        }

        void hotCorner_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            hotCorner.ReleaseMouseCapture();
            if (mouse.X < 0)
            {
                mouse = new Point(-pageWidth, pageHalfHeight);
            }
            else
            {
                mouse = new Point(pageWidth - 1, pageHalfHeight - 1);
            }
        }

        void hotCorner_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            mouse = e.GetPosition(Dots);
            follow = e.GetPosition(Dots);
            hotCorner.CaptureMouse();
        }

        void hotCorner_MouseMove(object sender, MouseEventArgs e)
        {
            Point testBounds = e.GetPosition(Dots);
            if (testBounds.X > pageWidth && testBounds.Y > pageHalfHeight)
            {
                hotCorner.ReleaseMouseCapture();
                mouse = new Point(pageWidth - 1, pageHalfHeight - 1);
            }
            else
            {
                mouse = e.GetPosition(Dots);
            }
        }

        Point spineTop;
        Point spineBottom;

        Point mouse = new Point(0, 0);
        Point follow = new Point(0, 0);
        Point radius1 = new Point(0, 0);
        Point radius2 = new Point(0, 0);
        Point bisector = new Point(0, 0);

        double dx = 0;
        double dy = 0;
        double a2f = 0;
        double fixedRadius = 0;
        double pageDiagonal;
        double distanceToFollow;

        Point corner = new Point(0, 0);
        Point tangentBottom = new Point(0, 0);
        Point maskSize = new Point(0, 0);
        Double bisectorAngle;
        Double bisectorTanget;
        Double tangentToCornerAngle;

        void updateUI()
        {
            Dots.Children.Clear();
            // THIS IS THE RAW FOLLOW
            follow.X += (mouse.X - follow.X) * .12;
            follow.Y += (mouse.Y - follow.Y) * .12;

            // CHECK DISTANCE FROM SPINE BOTTOM TO RAW FOLLOW
            dx = follow.X;
            dy = spineBottom.Y - follow.Y;
            // DETERMINE ANGLE FROM SPINE BOTTOM TO RAW FOLLOW
            a2f = Math.Atan2(dy, dx);
            // PLOT THE FIXED RADIUS FOLLOW
            radius1.X = Math.Cos(a2f) * fixedRadius;
            radius1.Y = spineBottom.Y - Math.Sin(a2f) * fixedRadius;
            // DETERMINE THE SHORTER OF THE TWO DISTANCES
            distanceToFollow = Math.Sqrt((spineBottom.Y - follow.Y) * (spineBottom.Y - follow.Y) + (follow.X * follow.X));
            double distToRadius1 = Math.Sqrt((spineBottom.Y - radius1.Y) * (spineBottom.Y - radius1.Y) + (radius1.X * radius1.X));
            // THE SMALLER OF THE TWO RADII DETERMINES THE CORNER
            if (distToRadius1 < distanceToFollow)
            {
                corner.X = radius1.X;
                corner.Y = radius1.Y;
            }
            else
            {
                corner.X = follow.X;
                corner.Y = follow.Y;
            }

            // NOW CHECK FOR THE OTHER CONSTRAINT, FROM THE SPINE TOP TO THE RADIUS OF THE PAGE DIAMETER...
            dx = spineTop.X - corner.X;
            dy = corner.Y + pageHalfHeight;
            distanceToFollow = Math.Sqrt(dx * dx + dy * dy);
            a2f = Math.Atan2(dy, dx);
            radius2.X = -Math.Cos(a2f) * pageDiagonal;
            radius2.Y = spineTop.Y + Math.Sin(a2f) * pageDiagonal;
            if (distanceToFollow > pageDiagonal) 
            {
                corner.X = radius2.X; 
                corner.Y = radius2.Y;
            }

            // CALCULATE THE BISECTOR AND CREATE THE CRITICAL TRIANGLE
            // DETERMINE THE MIDSECTION POINT
            bisector.X = corner.X + .5 * (pageWidth - corner.X);
            bisector.Y = corner.Y + .5 * (pageHalfHeight - corner.Y);
            bisectorAngle = Math.Atan2(pageHalfHeight - bisector.Y, pageWidth - bisector.X);
            bisectorTanget = bisector.X - Math.Tan(bisectorAngle) * (pageHalfHeight - bisector.Y);
            if (bisectorTanget < 0) bisectorTanget = 0;
            tangentBottom.X = bisectorTanget;
            tangentBottom.Y = pageHalfHeight;

            // DETERMINE THE tangentToCorner FOR THE ANGLE OF THE PAGE
            tangentToCornerAngle = Math.Atan2(tangentBottom.Y - corner.Y, tangentBottom.X - corner.X);

            // VISUALIZE THE ABOVE DATA

            drawLine(0, -pageHalfHeight, corner.X, corner.Y, red);
            drawLine(corner.X, corner.Y, pageWidth, pageHalfHeight, red);
            drawLine(0, pageHalfHeight, corner.X, corner.Y, red);
            drawLine(spineTop.X, spineTop.Y, corner.X, corner.Y, red);

            addDot(0, -pageHalfHeight, "ST");
            addDot(spineBottom.X, spineBottom.Y, "SB");
            addDot(pageWidth, pageHalfHeight, "EB");
            addDot(mouse.X, mouse.Y, "M");
            addDot(follow.X, follow.Y, "F");
            addDot(corner.X, corner.Y, "C");

            page0.X = origin.X + corner.X;
            page0.Y = origin.Y + corner.Y;
            page0.angle.Angle = tangentToCornerAngle * 180.0 / Math.PI;


            // VISUALIZE CRITICAL TRIANGLE
            drawLine(bisector.X, bisector.Y, bisector.X, pageHalfHeight, blue);
            drawLine(bisector.X, bisector.Y, tangentBottom.X, tangentBottom.Y, blue);
            drawLine(bisector.X, pageHalfHeight, tangentBottom.X, tangentBottom.Y, blue);
            addDot(bisector.X, bisector.Y, "T0");
            addDot(bisector.X, pageHalfHeight, "T1");
            addDot(tangentBottom.X, tangentBottom.Y, "T2");

            // DETERMINE THE ANGLE OF THE MAIN MASK RECTANGLE
            Double tanAngle = Math.Atan2(pageHalfHeight - bisector.Y, 
                                         bisector.X - bisectorTanget);

            // VISUALIZE THE CLIPPING RECTANGLE
            maskAngle.Angle = 90 * (tanAngle / Math.Abs(tanAngle)) 
                                    - tanAngle * 180 / Math.PI;
            mainMask.SetValue(Canvas.LeftProperty, 
                              pageWidth + (tangentBottom.X - maskSize.X));
            // WHEN YOU INEVITABLY CONVERT THIS FROM A RECTANGLE TO A 
            // RECTANGULAR CLIPPING PATH, YOU'LL NEED TO FIGURE OUT HOW TO 
            // CALCULATE THE CORNERS OF YOUR DYNAMIC RECTANGLE. YOU DO IT LIKE THIS:
            GeneralTransform tran = mainMask.TransformToVisual(Dots);
            addDot(tran.Transform(new Point(0, 0)), "R0");
            addDot(tran.Transform(new Point(mainMask.Width, 0)), "R1");
            addDot(tran.Transform(new Point(0, mainMask.Height)), "R2");
            addDot(tran.Transform(new Point(mainMask.Width, mainMask.Height)), "R3");



        }

        void addDot(double x, double y, string s)
        {
            Dot d = new Dot();
            d.X = x;
            d.Y = y;
            d.label.Text = s;
            Dots.Children.Add(d);
        }

        void addDot(Point p, string s)
        {
            Dot d = new Dot();
            d.X = p.X;
            d.Y = p.Y;
            d.label.Text = s;
            Dots.Children.Add(d);
        }

        void drawLine(double x1, double y1, double x2, double y2, Color c)
        {
            Line l = new Line();
            l.Stroke = new SolidColorBrush(c);
            l.X1 = x1;
            l.Y1 = y1;
            l.X2 = x2;
            l.Y2 = y2;
            Dots.Children.Add(l);
        }

    }
}
